@@ -1,5 +1,3 @@
-import six
-
 from .. import errors
 from ..constants import IS_WINDOWS_PLATFORM
 from ..utils import (
@@ -26,8 +24,8 @@ class TaskTemplate(dict):
         placement (Placement): Placement instructions for the scheduler.
             If a list is passed instead, it is assumed to be a list of
             constraints as part of a :py:class:`Placement` object.
-        networks (:py:class:`list`): List of network names or IDs to attach
-            the containers to.
+        networks (:py:class:`list`): List of network names or IDs or
+            :py:class:`NetworkAttachmentConfig` to attach the service to.
         force_update (int): A counter that triggers an update even if no
             relevant parameters have been changed.
     """
@@ -110,16 +108,23 @@ class ContainerSpec(dict):
         privileges (Privileges): Security options for the service's containers.
         isolation (string): Isolation technology used by the service's
             containers. Only used for Windows containers.
+        init (boolean): Run an init inside the container that forwards signals
+            and reaps processes.
+        cap_add (:py:class:`list`): A list of kernel capabilities to add to the
+            default set for the container.
+        cap_drop (:py:class:`list`): A list of kernel capabilities to drop from
+            the default set for the container.
     """
     def __init__(self, image, command=None, args=None, hostname=None, env=None,
                  workdir=None, user=None, labels=None, mounts=None,
                  stop_grace_period=None, secrets=None, tty=None, groups=None,
                  open_stdin=None, read_only=None, stop_signal=None,
                  healthcheck=None, hosts=None, dns_config=None, configs=None,
-                 privileges=None, isolation=None):
+                 privileges=None, isolation=None, init=None, cap_add=None,
+                 cap_drop=None):
         self['Image'] = image
 
-        if isinstance(command, six.string_types):
+        if isinstance(command, str):
             command = split_command(command)
         self['Command'] = command
         self['Args'] = args
@@ -149,7 +154,7 @@ class ContainerSpec(dict):
         if mounts is not None:
             parsed_mounts = []
             for mount in mounts:
-                if isinstance(mount, six.string_types):
+                if isinstance(mount, str):
                     parsed_mounts.append(Mount.parse_mount_string(mount))
                 else:
                     # If mount already parsed
@@ -183,6 +188,21 @@ class ContainerSpec(dict):
         if isolation is not None:
             self['Isolation'] = isolation
 
+        if init is not None:
+            self['Init'] = init
+
+        if cap_add is not None:
+            if not isinstance(cap_add, list):
+                raise TypeError('cap_add must be a list')
+
+            self['CapabilityAdd'] = cap_add
+
+        if cap_drop is not None:
+            if not isinstance(cap_drop, list):
+                raise TypeError('cap_drop must be a list')
+
+            self['CapabilityDrop'] = cap_drop
+
 
 class Mount(dict):
     """
@@ -219,7 +239,7 @@ class Mount(dict):
         self['Source'] = source
         if type not in ('bind', 'volume', 'tmpfs', 'npipe'):
             raise errors.InvalidArgument(
-                'Unsupported mount type: "{}"'.format(type)
+                f'Unsupported mount type: "{type}"'
             )
         self['Type'] = type
         self['ReadOnly'] = read_only
@@ -255,7 +275,7 @@ class Mount(dict):
         elif type == 'tmpfs':
             tmpfs_opts = {}
             if tmpfs_mode:
-                if not isinstance(tmpfs_mode, six.integer_types):
+                if not isinstance(tmpfs_mode, int):
                     raise errors.InvalidArgument(
                         'tmpfs_mode must be an integer'
                     )
@@ -275,7 +295,7 @@ class Mount(dict):
         parts = string.split(':')
         if len(parts) > 3:
             raise errors.InvalidArgument(
-                'Invalid mount format "{0}"'.format(string)
+                f'Invalid mount format "{string}"'
             )
         if len(parts) == 1:
             return cls(target=parts[0], source=None)
@@ -342,7 +362,7 @@ def _convert_generic_resources_dict(generic_resources):
             ' (found {})'.format(type(generic_resources))
         )
     resources = []
-    for kind, value in six.iteritems(generic_resources):
+    for kind, value in generic_resources.items():
         resource_type = None
         if isinstance(value, int):
             resource_type = 'DiscreteResourceSpec'
@@ -368,10 +388,11 @@ class UpdateConfig(dict):
 
         parallelism (int): Maximum number of tasks to be updated in one
           iteration (0 means unlimited parallelism). Default: 0.
-        delay (int): Amount of time between updates.
+        delay (int): Amount of time between updates, in nanoseconds.
         failure_action (string): Action to take if an updated task fails to
           run, or stops running during the update. Acceptable values are
-          ``continue`` and ``pause``. Default: ``continue``
+          ``continue``, ``pause``, as well as ``rollback`` since API v1.28.
+          Default: ``continue``
         monitor (int): Amount of time to monitor each updated task for
           failures, in nanoseconds.
         max_failure_ratio (float): The fraction of tasks that may fail during
@@ -385,9 +406,9 @@ class UpdateConfig(dict):
         self['Parallelism'] = parallelism
         if delay is not None:
             self['Delay'] = delay
-        if failure_action not in ('pause', 'continue'):
+        if failure_action not in ('pause', 'continue', 'rollback'):
             raise errors.InvalidArgument(
-                'failure_action must be either `pause` or `continue`.'
+                'failure_action must be one of `pause`, `continue`, `rollback`'
             )
         self['FailureAction'] = failure_action
 
@@ -413,7 +434,31 @@ class UpdateConfig(dict):
             self['Order'] = order
 
 
-class RestartConditionTypesEnum(object):
+class RollbackConfig(UpdateConfig):
+    """
+    Used to specify the way containe rollbacks should be performed by a service
+
+    Args:
+        parallelism (int): Maximum number of tasks to be rolled back in one
+          iteration (0 means unlimited parallelism). Default: 0
+        delay (int): Amount of time between rollbacks, in nanoseconds.
+        failure_action (string): Action to take if a rolled back task fails to
+          run, or stops running during the rollback. Acceptable values are
+          ``continue``, ``pause`` or ``rollback``.
+          Default: ``continue``
+        monitor (int): Amount of time to monitor each rolled back task for
+          failures, in nanoseconds.
+        max_failure_ratio (float): The fraction of tasks that may fail during
+          a rollback before the failure action is invoked, specified as a
+          floating point number between 0 and 1. Default: 0
+        order (string): Specifies the order of operations when rolling out a
+          rolled back task. Either ``start_first`` or ``stop_first`` are
+          accepted.
+    """
+    pass
+
+
+class RestartConditionTypesEnum:
     _values = (
         'none',
         'on-failure',
@@ -444,7 +489,7 @@ class RestartPolicy(dict):
                  max_attempts=0, window=0):
         if condition not in self.condition_types._values:
             raise TypeError(
-                'Invalid RestartPolicy condition {0}'.format(condition)
+                f'Invalid RestartPolicy condition {condition}'
             )
 
         self['Condition'] = condition
@@ -503,7 +548,7 @@ def convert_service_ports(ports):
         )
 
     result = []
-    for k, v in six.iteritems(ports):
+    for k, v in ports.items():
         port_spec = {
             'Protocol': 'tcp',
             'PublishedPort': k
@@ -623,18 +668,28 @@ class Placement(dict):
         Placement constraints to be used as part of a :py:class:`TaskTemplate`
 
         Args:
-            constraints (:py:class:`list`): A list of constraints
-            preferences (:py:class:`list`): Preferences provide a way to make
-                the scheduler aware of factors such as topology. They are
-                provided in order from highest to lowest precedence.
-            platforms (:py:class:`list`): A list of platforms expressed as
-                ``(arch, os)`` tuples
+            constraints (:py:class:`list` of str): A list of constraints
+            preferences (:py:class:`list` of tuple): Preferences provide a way
+                to make the scheduler aware of factors such as topology. They
+                are provided in order from highest to lowest precedence and
+                are expressed as ``(strategy, descriptor)`` tuples. See
+                :py:class:`PlacementPreference` for details.
+            maxreplicas (int): Maximum number of replicas per node
+            platforms (:py:class:`list` of tuple): A list of platforms
+                expressed as ``(arch, os)`` tuples
     """
-    def __init__(self, constraints=None, preferences=None, platforms=None):
+    def __init__(self, constraints=None, preferences=None, platforms=None,
+                 maxreplicas=None):
         if constraints is not None:
             self['Constraints'] = constraints
         if preferences is not None:
-            self['Preferences'] = preferences
+            self['Preferences'] = []
+            for pref in preferences:
+                if isinstance(pref, tuple):
+                    pref = PlacementPreference(*pref)
+                self['Preferences'].append(pref)
+        if maxreplicas is not None:
+            self['MaxReplicas'] = maxreplicas
         if platforms:
             self['Platforms'] = []
             for plat in platforms:
@@ -643,6 +698,27 @@ class Placement(dict):
                 })
 
 
+class PlacementPreference(dict):
+    """
+        Placement preference to be used as an element in the list of
+        preferences for :py:class:`Placement` objects.
+
+        Args:
+            strategy (string): The placement strategy to implement. Currently,
+                the only supported strategy is ``spread``.
+            descriptor (string): A label descriptor. For the spread strategy,
+                the scheduler will try to spread tasks evenly over groups of
+                nodes identified by this label.
+    """
+    def __init__(self, strategy, descriptor):
+        if strategy != 'spread':
+            raise errors.InvalidArgument(
+                'PlacementPreference strategy value is invalid ({}):'
+                ' must be "spread".'.format(strategy)
+            )
+        self['Spread'] = {'SpreadDescriptor': descriptor}
+
+
 class DNSConfig(dict):
     """
         Specification for DNS related configurations in resolver configuration
@@ -662,7 +738,7 @@ class DNSConfig(dict):
 
 
 class Privileges(dict):
-    """
+    r"""
         Security options for a service's containers.
         Part of a :py:class:`ContainerSpec` definition.
 
@@ -713,3 +789,21 @@ class Privileges(dict):
 
         if len(selinux_context) > 0:
             self['SELinuxContext'] = selinux_context
+
+
+class NetworkAttachmentConfig(dict):
+    """
+        Network attachment options for a service.
+
+        Args:
+            target (str): The target network for attachment.
+                Can be a network name or ID.
+            aliases (:py:class:`list`): A list of discoverable alternate names
+                for the service.
+            options (:py:class:`dict`): Driver attachment options for the
+                network target.
+    """
+    def __init__(self, target, aliases=None, options=None):
+        self['Target'] = target
+        self['Aliases'] = aliases
+        self['DriverOpts'] = options
