@@ -3,11 +3,10 @@ import shutil
 import unittest
 
 import docker
+from .. import helpers
 from docker.utils import kwargs_from_env
 
-from .. import helpers
-
-BUSYBOX = 'busybox:buildroot-2014.02'
+TEST_IMG = 'alpine:3.10'
 TEST_API_VERSION = os.environ.get('DOCKER_TEST_API_VERSION')
 
 
@@ -29,41 +28,44 @@ class BaseIntegrationTest(unittest.TestCase):
 
     def tearDown(self):
         client = docker.from_env(version=TEST_API_VERSION)
-        for img in self.tmp_imgs:
-            try:
-                client.api.remove_image(img)
-            except docker.errors.APIError:
-                pass
-        for container in self.tmp_containers:
-            try:
-                client.api.remove_container(container, force=True, v=True)
-            except docker.errors.APIError:
-                pass
-        for network in self.tmp_networks:
-            try:
-                client.api.remove_network(network)
-            except docker.errors.APIError:
-                pass
-        for volume in self.tmp_volumes:
-            try:
-                client.api.remove_volume(volume)
-            except docker.errors.APIError:
-                pass
+        try:
+            for img in self.tmp_imgs:
+                try:
+                    client.api.remove_image(img)
+                except docker.errors.APIError:
+                    pass
+            for container in self.tmp_containers:
+                try:
+                    client.api.remove_container(container, force=True, v=True)
+                except docker.errors.APIError:
+                    pass
+            for network in self.tmp_networks:
+                try:
+                    client.api.remove_network(network)
+                except docker.errors.APIError:
+                    pass
+            for volume in self.tmp_volumes:
+                try:
+                    client.api.remove_volume(volume)
+                except docker.errors.APIError:
+                    pass
 
-        for secret in self.tmp_secrets:
-            try:
-                client.api.remove_secret(secret)
-            except docker.errors.APIError:
-                pass
+            for secret in self.tmp_secrets:
+                try:
+                    client.api.remove_secret(secret)
+                except docker.errors.APIError:
+                    pass
 
-        for config in self.tmp_configs:
-            try:
-                client.api.remove_config(config)
-            except docker.errors.APIError:
-                pass
+            for config in self.tmp_configs:
+                try:
+                    client.api.remove_config(config)
+                except docker.errors.APIError:
+                    pass
 
-        for folder in self.tmp_folders:
-            shutil.rmtree(folder)
+            for folder in self.tmp_folders:
+                shutil.rmtree(folder)
+        finally:
+            client.close()
 
 
 class BaseAPIIntegrationTest(BaseIntegrationTest):
@@ -73,11 +75,11 @@ class BaseAPIIntegrationTest(BaseIntegrationTest):
     """
 
     def setUp(self):
-        super(BaseAPIIntegrationTest, self).setUp()
+        super().setUp()
         self.client = self.get_client_instance()
 
     def tearDown(self):
-        super(BaseAPIIntegrationTest, self).tearDown()
+        super().tearDown()
         self.client.close()
 
     @staticmethod
@@ -106,7 +108,7 @@ class BaseAPIIntegrationTest(BaseIntegrationTest):
 
         return container
 
-    def create_and_start(self, image=BUSYBOX, command='top', **kwargs):
+    def create_and_start(self, image=TEST_IMG, command='top', **kwargs):
         container = self.client.create_container(
             image=image, command=command, **kwargs)
         self.tmp_containers.append(container)
