@@ -1,11 +1,11 @@
 from docker.errors import APIError
-from .base import BaseAPIIntegrationTest, BUSYBOX
+from .base import BaseAPIIntegrationTest, TEST_IMG
 import pytest
 
 
 class ErrorsTest(BaseAPIIntegrationTest):
     def test_api_error_parses_json(self):
-        container = self.client.create_container(BUSYBOX, ['sleep', '10'])
+        container = self.client.create_container(TEST_IMG, ['sleep', '10'])
         self.client.start(container['Id'])
         with pytest.raises(APIError) as cm:
             self.client.remove_container(container['Id'])
