@@ -1,24 +1,22 @@
-# -*- coding: utf-8 -*-
-
 import datetime
 import json
 import signal
 
 import docker
+from docker.api import APIClient
 import pytest
-import six
 
 from . import fake_api
 from ..helpers import requires_api_version
 from .api_test import (
     BaseAPIClientTest, url_prefix, fake_request, DEFAULT_TIMEOUT_SECONDS,
-    fake_inspect_container
+    fake_inspect_container, url_base
 )
 
 try:
     from unittest import mock
 except ImportError:
-    import mock
+    from unittest import mock
 
 
 def fake_inspect_container_tty(self, container):
@@ -767,10 +765,71 @@ class CreateContainerTest(BaseAPIClientTest):
         assert args[1]['headers'] == {'Content-Type': 'application/json'}
         assert args[1]['timeout'] == DEFAULT_TIMEOUT_SECONDS
 
+    def test_create_container_with_device_requests(self):
+        client = APIClient(version='1.40')
+        fake_api.fake_responses.setdefault(
+            f'{fake_api.prefix}/v1.40/containers/create',
+            fake_api.post_fake_create_container,
+        )
+        client.create_container(
+            'busybox', 'true', host_config=client.create_host_config(
+                device_requests=[
+                    {
+                        'device_ids': [
+                            '0',
+                            'GPU-3a23c669-1f69-c64e-cf85-44e9b07e7a2a'
+                        ]
+                    },
+                    {
+                        'driver': 'nvidia',
+                        'Count': -1,
+                        'capabilities': [
+                            ['gpu', 'utility']
+                        ],
+                        'options': {
+                            'key': 'value'
+                        }
+                    }
+                ]
+            )
+        )
+
+        args = fake_request.call_args
+        assert args[0][1] == url_base + 'v1.40/' + 'containers/create'
+        expected_payload = self.base_create_payload()
+        expected_payload['HostConfig'] = client.create_host_config()
+        expected_payload['HostConfig']['DeviceRequests'] = [
+            {
+                'Driver': '',
+                'Count': 0,
+                'DeviceIDs': [
+                    '0',
+                    'GPU-3a23c669-1f69-c64e-cf85-44e9b07e7a2a'
+                ],
+                'Capabilities': [],
+                'Options': {}
+            },
+            {
+                'Driver': 'nvidia',
+                'Count': -1,
+                'DeviceIDs': [],
+                'Capabilities': [
+                    ['gpu', 'utility']
+                ],
+                'Options': {
+                    'key': 'value'
+                }
+            }
+        ]
+        assert json.loads(args[1]['data']) == expected_payload
+        assert args[1]['headers']['Content-Type'] == 'application/json'
+        assert set(args[1]['headers']) <= {'Content-Type', 'User-Agent'}
+        assert args[1]['timeout'] == DEFAULT_TIMEOUT_SECONDS
+
     def test_create_container_with_labels_dict(self):
         labels_dict = {
-            six.text_type('foo'): six.text_type('1'),
-            six.text_type('bar'): six.text_type('2'),
+            'foo': '1',
+            'bar': '2',
         }
 
         self.client.create_container(
@@ -786,12 +845,12 @@ class CreateContainerTest(BaseAPIClientTest):
 
     def test_create_container_with_labels_list(self):
         labels_list = [
-            six.text_type('foo'),
-            six.text_type('bar'),
+            'foo',
+            'bar',
         ]
         labels_dict = {
-            six.text_type('foo'): six.text_type(),
-            six.text_type('bar'): six.text_type(),
+            'foo': '',
+            'bar': '',
         }
 
         self.client.create_container(
@@ -951,11 +1010,11 @@ class CreateContainerTest(BaseAPIClientTest):
 
     def test_create_container_with_unicode_envvars(self):
         envvars_dict = {
-            'foo': u'☃',
+            'foo': '☃',
         }
 
         expected = [
-            u'foo=☃'
+            'foo=☃'
         ]
 
         self.client.create_container(
@@ -1076,7 +1135,7 @@ class ContainerTest(BaseAPIClientTest):
             stream=False
         )
 
-        assert logs == 'Flowering Nights\n(Sakuya Iyazoi)\n'.encode('ascii')
+        assert logs == b'Flowering Nights\n(Sakuya Iyazoi)\n'
 
     def test_logs_with_dict_instead_of_id(self):
         with mock.patch('docker.api.client.APIClient.inspect_container',
@@ -1092,7 +1151,7 @@ class ContainerTest(BaseAPIClientTest):
             stream=False
         )
 
-        assert logs == 'Flowering Nights\n(Sakuya Iyazoi)\n'.encode('ascii')
+        assert logs == b'Flowering Nights\n(Sakuya Iyazoi)\n'
 
     def test_log_streaming(self):
         with mock.patch('docker.api.client.APIClient.inspect_container',
