@@ -1,14 +1,12 @@
 import json
 
-import six
-
 from .api_test import BaseAPIClientTest, url_prefix, response
 from docker.types import IPAMConfig, IPAMPool
 
 try:
     from unittest import mock
 except ImportError:
-    import mock
+    from unittest import mock
 
 
 class NetworkTest(BaseAPIClientTest):
@@ -103,16 +101,16 @@ class NetworkTest(BaseAPIClientTest):
             self.client.remove_network(network_id)
 
         args = delete.call_args
-        assert args[0][0] == url_prefix + 'networks/{0}'.format(network_id)
+        assert args[0][0] == url_prefix + f'networks/{network_id}'
 
     def test_inspect_network(self):
         network_id = 'abc12345'
         network_name = 'foo'
         network_data = {
-            six.u('name'): network_name,
-            six.u('id'): network_id,
-            six.u('driver'): 'bridge',
-            six.u('containers'): {},
+            'name': network_name,
+            'id': network_id,
+            'driver': 'bridge',
+            'containers': {},
         }
 
         network_response = response(status_code=200, content=network_data)
@@ -123,7 +121,7 @@ class NetworkTest(BaseAPIClientTest):
             assert result == network_data
 
         args = get.call_args
-        assert args[0][0] == url_prefix + 'networks/{0}'.format(network_id)
+        assert args[0][0] == url_prefix + f'networks/{network_id}'
 
     def test_connect_container_to_network(self):
         network_id = 'abc12345'
@@ -136,11 +134,12 @@ class NetworkTest(BaseAPIClientTest):
                 container={'Id': container_id},
                 net_id=network_id,
                 aliases=['foo', 'bar'],
-                links=[('baz', 'quux')]
+                links=[('baz', 'quux')],
+                driver_opt={'com.docker-py.setting': 'yes'},
             )
 
         assert post.call_args[0][0] == (
-            url_prefix + 'networks/{0}/connect'.format(network_id)
+            url_prefix + f'networks/{network_id}/connect'
         )
 
         assert json.loads(post.call_args[1]['data']) == {
@@ -148,6 +147,7 @@ class NetworkTest(BaseAPIClientTest):
             'EndpointConfig': {
                 'Aliases': ['foo', 'bar'],
                 'Links': ['baz:quux'],
+                'DriverOpts': {'com.docker-py.setting': 'yes'},
             },
         }
 
@@ -162,7 +162,7 @@ class NetworkTest(BaseAPIClientTest):
                 container={'Id': container_id}, net_id=network_id)
 
         assert post.call_args[0][0] == (
-            url_prefix + 'networks/{0}/disconnect'.format(network_id)
+            url_prefix + f'networks/{network_id}/disconnect'
         )
         assert json.loads(post.call_args[1]['data']) == {
             'Container': container_id
