@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import unittest
 
 import pytest
@@ -14,8 +12,8 @@ from docker.types.services import convert_service_ports
 
 try:
     from unittest import mock
-except:
-    import mock
+except:  # noqa: E722
+    from unittest import mock
 
 
 def create_host_config(*args, **kwargs):
@@ -85,6 +83,12 @@ class HostConfigTest(unittest.TestCase):
         with pytest.raises(ValueError):
             create_host_config(version='1.23', userns_mode='host12')
 
+    def test_create_host_config_with_uts(self):
+        config = create_host_config(version='1.15', uts_mode='host')
+        assert config.get('UTSMode') == 'host'
+        with pytest.raises(ValueError):
+            create_host_config(version='1.15', uts_mode='host12')
+
     def test_create_host_config_with_oom_score_adj(self):
         config = create_host_config(version='1.22', oom_score_adj=100)
         assert config.get('OomScoreAdj') == 100
